@@ -8,7 +8,7 @@ import pathlib
 import xml.etree.ElementTree
 import numpy as np
 from typing import List, Union, Tuple
-from aston.tracefile.agilent_uv import AgilentCSDAD2
+from aston.tracefile.agilent_uv import AgilentCSDAD2, string_read
 from aston.tracefile.agilent_ms import AgilentMS
 from unithandler.base import UnitFloat
 
@@ -89,6 +89,48 @@ def bisect_slice(array, minimum_value: float, maximum_value: float) -> Tuple[int
     )
 
 
+def check_or_locate_file(path: Union[str, pathlib.Path], file_name: str) -> pathlib.Path:
+    """
+    Checks whether the provided path points to the provided file name. If not, checks whether the path is a directory
+    and searches for the file in the directory. If there are multiple occurences of the provided file name in a
+    directory, the first is returned.
+
+    :param path: path to search
+    :param file_name: target file name
+    :return: path to desired file
+    """
+    if isinstance(path, pathlib.Path) is False:
+        path = pathlib.Path(path)
+    if path.name.lower() == file_name.lower():
+        return path
+    elif path.is_dir():
+        try:
+            return next(path.glob(file_name))
+        except StopIteration:
+            raise FileNotFoundError(f'no file name matching "{file_name}" could be found in path {path}')
+    raise FileNotFoundError(f'the path {path} does not point to a "{file_name}" file')
+
+
+def retrieve_metadata_from_channel(path: Union[str, pathlib.Path]) -> dict:
+    """
+    Retrieves metadata from a .CH file
+
+    :param path: path to read
+    :return: returns a dictionary containing metadata from the channel
+    """
+    with open(path, 'rb') as f:
+        # read in signal information string
+        f.seek(0x1075)
+        signal = string_read(f)
+        # read chemstation version
+        f.seek(0xEE3)
+        cs_version = string_read(f)
+    return {
+        'signal_spec': signal,
+        'cs_version': cs_version,
+    }
+
+
 class DADSignalInfo(object):
     # default unit for wavelength values
     DEFAULT_WAVELENGTH_UNIT = 'nm'
@@ -252,13 +294,40 @@ class DADSignalInfo(object):
             parsed['name'] = name_override
         return cls(**parsed)
 
+    @classmethod
+    def create_from_CH_file(cls, file_path: Union[str, pathlib.Path]) -> "DADSignalInfo":
+        """
+        Creates a DADSignal info instance from a channel file.
+
+        :param file_path: target file path
+        """
+        vals = retrieve_metadata_from_channel(file_path)
+        return cls.create_from_agilent_string(vals['signal_spec'])
+
+    @classmethod
+    def get_signals_in_directory(cls, file_path: Union[str, pathlib.Path]) -> List["DADSignalInfo"]:
+        """
+        Creates a list of signals based on the .CH files in a directory.
+
+        :param file_path: path to target directory
+        :return: list of signal info objects
+        """
+        if isinstance(file_path, pathlib.Path) is False:
+            file_path = pathlib.Path(file_path)
+        if file_path.is_dir() is False:
+            raise ValueError(f'the provided path is not a directory: {file_path}')
+        out = []
+        for ch_file in file_path.glob('*.CH'):
+            out.append(cls.create_from_CH_file(ch_file))
+        return out
+
 
 class HPLCSampleInfo(object):
     def __init__(self,
                  sample_file_name: str,
-                 datetimestamp: Union[str, datetime.datetime],
                  method_name: str,
                  signals: Union[List[DADSignalInfo], List[str]],
+                 datetimestamp: Union[str, datetime.datetime] = None,
                  ):
         """
         Data class for describing an HPLC sample.
@@ -343,20 +412,14 @@ class HPLCSampleInfo(object):
         return xml.etree.ElementTree.parse(acaml_path)
 
     @classmethod
-    def get_values_from_xml(cls, xml_path: Union[str, pathlib.Path]) -> dict:
+    def get_values_from_result_xml(cls, xml_path: Union[str, pathlib.Path]) -> dict:
         """
-        Attempts to find a Result.xml file and parse sample information from that. This is an old-style ChemStation
+        Retrieves values from a Result.xml file. This is an old-style ChemStation
         metadata file (~B.04.03 era).
 
         :param xml_path: path to xml or directory containing xml file
         """
-        if isinstance(xml_path, pathlib.Path) is False:
-            xml_path = pathlib.Path(xml_path)
-        if xml_path.is_dir():
-            try:
-                xml_path = list(xml_path.glob('Result.xml'))[0]
-            except IndexError:
-                raise FileNotFoundError(f'no Result.xml files could be found in the directory {xml_path}')
+        xml_path = check_or_locate_file(xml_path, 'Result.xml')
         tree = xml.etree.ElementTree.parse(xml_path)
         root = tree.getroot()
 
@@ -378,6 +441,45 @@ class HPLCSampleInfo(object):
             'signals': signals,
         }
 
+    @classmethod
+    def get_values_from_sample_xml(cls, xml_path: Union[str, pathlib.Path]) -> dict:
+        """
+        Retrieves values from a Sample.xml file. From ChemStation C.01.07
+
+        :param xml_path: path to xml or directory containing xml file
+        """
+        xml_path = check_or_locate_file(xml_path, 'Sample.xml')
+        tree = xml.etree.ElementTree.parse(xml_path)
+        root = tree.getroot()
+        sample_name = next(root.iter('Name')).text
+        method_path = pathlib.Path(next(root.iter('ACQMethodPath')).text)
+        signals = DADSignalInfo.get_signals_in_directory(
+            xml_path.parent
+        )
+        return {
+            'sample_file_name': sample_name,
+            'method_name': method_path.name,
+            'signals': signals,
+        }
+
+    @classmethod
+    def get_values_from_xml(cls, xml_path: Union[str, pathlib.Path]) -> dict:
+        """
+        Attempts to find a Result.xml file and parse sample information from that.
+
+        :param xml_path: path to xml or directory containing xml file
+        """
+        # todo make this not terrible
+        try:
+            return cls.get_values_from_sample_xml(xml_path)
+        except FileNotFoundError:
+            pass
+        try:
+            return cls.get_values_from_result_xml(xml_path)
+        except FileNotFoundError:
+            pass
+        raise FileNotFoundError(f'no known sample metadata files could be found in the path {xml_path}')
+
     @classmethod
     def get_values_from_acaml(cls,
                               acaml: Union[
@@ -1063,9 +1165,9 @@ class DADSignal(DADSignalInfo):
 class HPLCSample(HPLCSampleInfo):
     def __init__(self,
                  sample_file_name: str,
-                 datetimestamp: Union[str, datetime.datetime],
                  method_name: str,
                  signals: Union[List[DADSignalInfo], List[DADSignal], List[str]],
+                 datetimestamp: Union[str, datetime.datetime] = None,
                  dad_spectrum: DADSpectrum = None,
                  ms_spectrum: MSSpectrum = None,
                  directory: str = None,
@@ -1083,9 +1185,9 @@ class HPLCSample(HPLCSampleInfo):
         HPLCSampleInfo.__init__(
             self,
             sample_file_name=sample_file_name,
-            datetimestamp=datetimestamp,
             method_name=method_name,
             signals=signals,
+            datetimestamp=datetimestamp
         )
         self.spectrum: DADSpectrum = dad_spectrum
         self.ms_spectrum: MSSpectrum = ms_spectrum
